/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import java.util.ArrayList;
import java.util.List;

import org.conqat.lib.commons.collections.CollectionUtils;
import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Ellipse;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.XYLayout;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.swt.graphics.Color;
import org.fortiss.tooling.base.model.element.IConnection;
import org.fortiss.tooling.base.model.element.IConnector;
import org.fortiss.tooling.base.model.element.IHierarchicElement;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.ui.editpart.figure.AntiAliasedEllipse;
import org.fortiss.tooling.base.ui.editpart.policy.ConnectionGraphicalNodeEditPolicy;
import org.fortiss.tooling.base.ui.editpart.policy.RemoveEditPolicy;
import org.fortiss.tooling.base.ui.layout.constants.RecommendedLayoutConstants;
import org.fortiss.tooling.base.ui.layout.util.LayoutDataUtils;

/**
 * Base class for connector edit parts.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 33691B003B390F6576C82DFB32395F1C
 */
public abstract class ConnectorEditPartBase<T extends ILayoutedModelElement & IConnector>
		extends PositionedEditPartBase<T> {

	/** Constructor. */
	protected ConnectorEditPartBase(T modelElement) {
		super(modelElement);
	}

	/** {@inheritDoc} */
	@Override
	protected Rectangle determinePosition() {
		Rectangle r = LayoutDataUtils.getNodeBounds(modelElement);
		r.setSize(RecommendedLayoutConstants.DEFAULT_CONNECTOR_DIMENSION);
		return r;
	}

	/** {@inheritDoc} */
	@Override
	protected IFigure createBaseFigure() {
		Figure base = new Figure();
		base.setLayoutManager(new XYLayout());
		Ellipse connector = new AntiAliasedEllipse();
		base.add(connector,
				RecommendedLayoutConstants.DEFAULT_CONNECTOR_FIGURE_BOUNDS);
		if (isTarget()) {
			connector.setBackgroundColor(getTargetBackgroundColor());
			connector.setForegroundColor(getTargetForegroundColor());
		} else {
			connector.setBackgroundColor(getSourceBackgroundColor());
			connector.setForegroundColor(getSourceForegroundColor());
		}
		return base;
	}

	/** Returns background color for target role. */
	protected Color getTargetForegroundColor() {
		return getForegroundColor();
	}

	/** Returns background color for target role. */
	protected Color getTargetBackgroundColor() {
		return getBackgroundColor();
	}

	/** Returns background color for source role. */
	protected Color getSourceForegroundColor() {
		return getForegroundColor();
	}

	/** Returns background color for source role. */
	protected Color getSourceBackgroundColor() {
		return getForegroundColor();
	}

	/** {@inheritDoc} */
	@Override
	protected Color getForegroundColor() {
		return ColorConstants.black;
	}

	/** {@inheritDoc} */
	@Override
	protected Color getBackgroundColor() {
		return ColorConstants.white;
	}

	/** {@inheritDoc} */
	@Override
	protected ConnectionAnchor defaultConnectionAnchor() {
		return new ChopboxAnchor(baseFigure);
	}

	/** {@inheritDoc} */
	@SuppressWarnings({ "unchecked" })
	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();

		refreshSourceConnections();
		refreshTargetConnections();

		for (ConnectionEditPart editPart : (List<ConnectionEditPart>) getSourceConnections()) {
			editPart.refresh();
		}
		for (ConnectionEditPart editPart : (List<ConnectionEditPart>) getTargetConnections()) {
			editPart.refresh();
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected List<IConnection> getModelSourceConnections() {
		if (!isTarget()) {
			return filterConnections(modelElement.getOutgoingList(),
					getRootHierarchicElement());
		}
		return CollectionUtils.emptyList();
	}

	/** {@inheritDoc} */
	@Override
	protected List<IConnection> getModelTargetConnections() {
		if (isTarget()) {
			return filterConnections(modelElement.getIncomingList(),
					getRootHierarchicElement());
		}
		return CollectionUtils.emptyList();
	}

	/** Filters any channels not owned by the given owner. */
	private List<IConnection> filterConnections(List<IConnection> connections,
			IHierarchicElement owner) {
		List<IConnection> result = new ArrayList<IConnection>();
		for (IConnection connection : connections) {
			if (connection.eContainer() == owner) {
				result.add(connection);
			}
		}
		return result;
	}

	/** {@inheritDoc} */
	@Override
	protected void createEditPolicies() {
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new RemoveEditPolicy());

		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE,
				new ConnectionGraphicalNodeEditPolicy());
	}

	/** Returns the root element for this connector edit part. */
	protected abstract IHierarchicElement getRootHierarchicElement();

	/**
	 * Returns whether the underlying connector edit part acts as a target
	 * connector in the current view (i.e. edit part).
	 */
	protected abstract boolean isTarget();
}
